/*
 * MESSAGES.H
 * defines the global functions found in MESSAGES.C
 */


/*
 * ALERT_DEMOS:
 * prints out an alert message, but performs no other actions.
 */
extern void alert_demos();


/*
 * ABORT_DEMOS:
 * prints out an error message and aborts the program.
 */
extern void abort_demos();


/*
 * USAGE:
 * describes the correct usage of this program.
 */
extern void usage();
