/**********************************************************************
**
** agents.h - include file for modules need Agent information
**
**********************************************************************/

/*
 * $Log: agents.h,v $
 * Revision 1.1  1991/10/11  23:15:59  jefft
 * Initial revision
 *
 */


typedef struct LISTELEM {
  struct LISTELEM       *next;
  char                  *alias;
  char                  *path;
  char                  *args;
  char			*pixmap;
  char                  **inports;
  char                  **outports;
  }  AGENT;


extern void	addAgent();
extern AGENT	*findAgent();

