
typedef union  { int y_int; char *y_str; char **y_ptr;} YYSTYPE;
extern YYSTYPE yylval;
# define NAME 257
# define AOP 258
# define IOP 259
# define COP 260
# define CSYM 261
# define PORTSEP 262
# define Sconstant 263
