/*
	Author	: Sreedhar Sivakumaran
	Purpose	: Missing library functions. 
	Notes	: 
		1. strdup not available in dec ( yahoo ). 
*/

#if	defined(host_mips)
#include <malloc.h>

char	*strdup( s1 )
char	*s1 ;
{
char	*s2 ;

	if( s2 = malloc(strlen(s1) + 1) ) strcpy( s2, s1 );
	return s2 ;
}
#endif
