/**********************************************************************
**
** route.h - include file for modules needing access to routing tables
**
**********************************************************************/

/*
 * $Log: route.h,v $
 * Revision 1.1  1991/10/11  23:16:08  jefft
 * Initial revision
 *
 */

typedef enum {
  INITIAL, PENDING, CONNECT
  } RSTATUS;


typedef struct R {
  struct R	*next;
  RSTATUS	status;
  char		*outport;
  char		**destination;
  } ROUTE;


extern ROUTE *makeRouting();
extern void addRoute();
extern void deleteRoute();
extern int backgroundRouting();

